package com.loafle.overflow.container.client.handler;

import com.loafle.commons.rpc.RPCException;
import com.loafle.commons.rpc.protocol.RPCClientCodec;
import com.loafle.commons.rpc.protocol.RPCClientResponseCodec;
import com.loafle.commons.rpc.registry.RPCInvoker;
import com.loafle.overflow.container.Container;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.codec.http.websocketx.TextWebSocketFrame;
import io.netty.handler.codec.http.websocketx.WebSocketFrame;

/**
 * RPCClientHandler
 */
public class RPCClientHandler extends SimpleChannelInboundHandler<WebSocketFrame> {
  private static final Logger logger = LoggerFactory.getLogger(RPCClientHandler.class);

  @Autowired
  @Qualifier(Container.RPC_CLIENT_CODEC)
  private RPCClientCodec clientCodec;
  @Autowired()
  @Qualifier(Container.RPC_INVOKER)
  private RPCInvoker rpcInvoker;

  private ChannelHandlerContext ctx;

	@Override
	protected void channelRead0(ChannelHandlerContext ctx, WebSocketFrame frame) throws Exception {
    if (frame instanceof TextWebSocketFrame) {
      ByteBuf buff = null;
      byte[] responseBytes = null;
      RPCClientResponseCodec responseCodec = null;
      try {
        buff = frame.content();
        responseBytes = new byte[buff.readableBytes()];
        buff.getBytes(buff.readerIndex(), responseBytes);

        responseCodec = this.clientCodec.response(responseBytes);
        if (responseCodec.isNotification()) {
          this.rpcInvoker.invoke(responseCodec.notification());
        } else {
          throw new UnsupportedOperationException(String.format("request is not unsupported"));
        }
      } catch (RPCException e) {
        logger.debug("Error", e);
        throw e; 
      }
    } else {
      throw new UnsupportedOperationException(String.format("unsupported frame type: %s", frame.getClass().getName()));
    }
  }
  
  public void send(String method, Object... params) {
    if (null == this.ctx || !this.ctx.channel().isOpen()) {
      logger.warn("ChannelHandlerContext is not valid");
      return;
    }
    try {
      byte[] requestBytes = this.clientCodec.request(method, params, null);
      this.ctx.channel().writeAndFlush(requestBytes);
    } catch (RPCException e) {
      logger.error("cannot create notification", e);
    }
  }

  @Override
  public void channelRegistered(ChannelHandlerContext ctx) throws Exception {
    // Initialize early if channel is active already.
    if (ctx.channel().isActive()) {
      this.ctx = ctx;
    }
    super.channelRegistered(ctx);
  }

  @Override
  public void channelActive(ChannelHandlerContext ctx) throws Exception {
    this.ctx = ctx;
    super.channelActive(ctx);
  }

  /**
   * Calls {@link ChannelHandlerContext#fireChannelInactive()} to forward
   * to the next {@link ChannelInboundHandler} in the {@link ChannelPipeline}.
   *
   * Sub-classes may override this method to change behavior.
   */
  @Override
  public void channelInactive(ChannelHandlerContext ctx) throws Exception {
    this.ctx = null;
    super.channelInactive(ctx);
  }
}