package com.loafle.overflow.container.configuration;

import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.List;

import javax.annotation.Resource;

import com.loafle.commons.rpc.protocol.RPCClientCodec;
import com.loafle.commons.rpc.protocol.json.JSONRPCClientCodec;
import com.loafle.commons.rpc.registry.RPCRegistry;
import com.loafle.commons.server.websocket.client.Client;
import com.loafle.overflow.config.container.ContainerProtocol;
import com.loafle.overflow.container.Container;
import com.loafle.overflow.container.client.handler.RPCClientHandler;

import org.codehaus.jackson.map.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;

import io.netty.channel.Channel;
import io.netty.channel.ChannelHandler;
import io.netty.channel.socket.nio.NioSocketChannel;

/**
 * ContainerConfiguration
 */
@Configuration
@ComponentScan(basePackages = { "com.loafle.overflow" })
@PropertySource({ "classpath:netty.properties" })
public class ContainerConfiguration {
  @Resource(name=Container.PORT_NUMBER)
  private int portNumber;

  @Autowired()
  private ObjectMapper objectMapper;

  @Autowired(required=false)
  @Qualifier(Container.CHANNEL_HANDLERS)
  private List<ChannelHandler> extraChannelHandlers;

  @Bean(Client.CHANNEL_CLASS)
  public Class<? extends Channel> channelClass() {
    return NioSocketChannel.class;
  }

  @Bean(Client.CHANNEL_HANDLERS)
  public List<ChannelHandler> channelHandlers() {
    List<ChannelHandler> chs = Arrays.asList(this.rpcClientHandler());
    if (null != this.extraChannelHandlers) {
      chs.addAll(this.extraChannelHandlers);
    }

    return chs;
  }

  @Bean(Client.SERVER_URI)
  public URI probeURI() throws URISyntaxException {
    return new URI(String.format("ws://127.0.0.1:%d%s", portNumber, ContainerProtocol.HTTPEntry_Container));
  }

  @Bean(Container.RPC_CLIENT_CODEC)
  public RPCClientCodec rpcClientCodec() {
    return new JSONRPCClientCodec(this.objectMapper);
  }

  @Bean(Container.RPC_CLIENT_CODEC)
  public RPCClientHandler rpcClientHandler() {
    return new RPCClientHandler();
  }

  @Bean({Container.RPC_INVOKER, Container.RPC_REGISTRY})
  public RPCRegistry rpcRegistry() {
    return new RPCRegistry();
  }

}